require "binary_search.rb"
require "test/unit"

class TestBinarySearch < Test::Unit::TestCase	
	def test_empty
		assert_nil(binary_search([], 1), "Should return nil for empty array")
		assert_nil(binary_search([], "blah"), "Should return nil for empty array")
	end
	
	def test_singleton
		assert_equal(0, binary_search([1], 1))
		assert_nil(binary_search([-1], 17), "Shouldn't find value")
	end
	
	def test_containing
		# test all entries in a sorted array
		sorted = (1..100).entries
		1.upto(100).each{|i| assert_equal(i-1, binary_search(sorted, i))}
		
		sorted.map!{|val| val * 10}		
		sorted.each do |i|
			index = binary_search(sorted, i)
			refute_nil(index, "Should have found the value in the array")
			assert_equal(i, sorted[index])
		end
	end
	
	def test_containing_strings
		# test all entries in a sorted array
		sorted = ("a".."z").entries
		"a".upto("z").each_with_index{|val, index| assert_equal(binary_search(sorted, val), index)}
		
		findme = "asdfjklm"
		sorted.select!{|val| val =~ /^[#{findme}]/}	
		findme.each_char do |ch|
			index = binary_search(sorted, ch)
			refute_nil(index, "Should have found the value in the array")
			assert_equal(ch, sorted[index])
		end
	end
	
	def test_not_containing
		# test all entries in a sorted array
		sorted = (1..100).entries
		
		# try a few random numbers that aren't 100
		10.times{assert_nil(binary_search(sorted, rand(10000)+100), "Shouldn't find value")}		
	end
end